#include "OrcCBindings.h"
#include "llvm-c/Target.h"
#include "llvm-c/TargetMachine.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

typedef int (*MainFnTy)(void);

LLVMTargetMachineRef getHostTargetMachine(void) {
  char *TgtTriple = LLVMGetDefaultTargetTriple();

  char *ErrMsg = 0;
  LLVMTargetRef Tgt;
  if (LLVMGetTargetFromTriple(TgtTriple, &Tgt, &ErrMsg)) {
    fprintf(stderr, "Error - Could not create target: '%s'.\n", ErrMsg);
    exit(1);
  }

  return LLVMCreateTargetMachine(Tgt, TgtTriple, "", "", LLVMCodeGenLevelLess,
                                 LLVMRelocDefault, LLVMCodeModelLarge);
}

static void myTestFuncImpl(void) {
  printf("Hello, World!\n");
}

uint64_t myResolver(const char *Name, void *Ctx) {
  if (!strncmp(Name, "testFunc", 8))
    return (uint64_t)&myTestFuncImpl;
  return 0;
}

int main(int argc, char *argv[]) {
  LLVMInitializeNativeTarget();
  LLVMInitializeNativeAsmPrinter();
  LLVMInitializeNativeAsmParser();

  LLVMModuleRef Module = LLVMModuleCreateWithName("Mod");

  LLVMTypeRef VoidFnType = LLVMFunctionType(LLVMVoidType(), NULL, 0, 0);
  LLVMValueRef TestFn = LLVMAddFunction(Module, "testFunc", VoidFnType);

  LLVMTypeRef MainFnType = LLVMFunctionType(LLVMInt32Type(), NULL, 0, 0);
  LLVMValueRef MainFn = LLVMAddFunction(Module, "main", MainFnType);
  LLVMSetLinkage(MainFn, LLVMExternalLinkage);

  LLVMBasicBlockRef Entry = LLVMAppendBasicBlock(MainFn, "");
  LLVMBuilderRef Builder = LLVMCreateBuilder();

  LLVMPositionBuilderAtEnd(Builder, Entry);
  LLVMBuildCall(Builder, TestFn, NULL, 0, "");
  LLVMBuildRet(Builder, LLVMConstInt(LLVMInt32Type(), 0, 0));

  OrcJITStackRef JIT = OrcCreateInstance(getHostTargetMachine(),
                                         LLVMGetGlobalContext());

  OrcAddModuleLazy(JIT, Module, myResolver, 0);
  MainFnTy MainFnPtr = (MainFnTy)OrcGetSymbolAddress(JIT, "main");
  int Result = MainFnPtr();

  OrcDisposeInstance(JIT);

  return Result;
}
