#ifndef ORCCBINDINGS_H
#define ORCCBINDINGS_H

#include "llvm-c/Support.h"
#include "llvm-c/TargetMachine.h"

#ifdef __cplusplus
extern "C" {
#endif

typedef struct OrcOpaqueJITStack *OrcJITStackRef;
typedef uint64_t (*OrcExternalSymbolResolverFn)(const char *Name,
                                                void *LookupCtx);
typedef uint64_t (*OrcLazyCompileCallback)(void *CompileCallbackCtx);

/**
 * Create an ORC JIT stack.
 *
 * The client owns the resulting stack, and must call OrcDisposeInstance(...)
 * to destroy it and free its memory. The JIT stack will take ownership of the
 * TargetMachine, which will be destroyed when the stack is destroyed. The
 * client should not attempt to dispose of the Target Machine, or it will result
 * in a double-free.
 */
OrcJITStackRef OrcCreateInstance(LLVMTargetMachineRef TM,
                                 LLVMContextRef Context);

/**
 * Add module to be lazily compiled.
 */
void OrcAddModuleLazy(OrcJITStackRef JITStack, LLVMModuleRef Mod,
                      OrcExternalSymbolResolverFn ExternalResolver,
                      void *ExternalResolverCtx);

/**
 * Get symbol address from JIT instance.
 */
void* OrcGetSymbolAddress(OrcJITStackRef JITStack, const char *SymbolName);

/**
 * Dispose of an ORC JIT stack.
 */
void OrcDisposeInstance(OrcJITStackRef JITStack);

#ifdef __cplusplus
}
#endif /* extern "C" */

#endif /* ORCCBINDINGS_H */
