#include "OrcCBindings.h"
#include "OrcCBindingsStack.h"

using namespace llvm;

DEFINE_SIMPLE_CONVERSION_FUNCTIONS(TargetMachine, LLVMTargetMachineRef);

OrcJITStackRef OrcCreateInstance(LLVMTargetMachineRef TM,
                                 LLVMContextRef Context) {
  std::unique_ptr<TargetMachine> TM2(unwrap(TM));
  LLVMContext &Ctx = *unwrap(Context);

  auto CallbackMgrBuilder =
    OrcCBindingsStack::createCallbackManagerBuilder(
      Triple(TM2->getTargetTriple()));

  OrcCBindingsStack *JITStack =
    new OrcCBindingsStack(std::move(TM2), Ctx, CallbackMgrBuilder);

  return wrap(JITStack);
}

void OrcAddModuleLazy(OrcJITStackRef JITStack, LLVMModuleRef Mod,
                      OrcExternalSymbolResolverFn ExternalResolver,
                      void *ExternalResolverCtx) {
  OrcCBindingsStack &J = *unwrap(JITStack);
  std::unique_ptr<Module> M(unwrap(Mod));
  J.addModule(std::move(M), ExternalResolver, ExternalResolverCtx);
}

void* OrcGetSymbolAddress(OrcJITStackRef JITStack, const char *SymbolName) {
  OrcCBindingsStack &J = *unwrap(JITStack);
  auto Sym = J.findSymbol(SymbolName);
  return reinterpret_cast<void*>(static_cast<uintptr_t>(Sym.getAddress()));
}

void OrcDisposeInstance(OrcJITStackRef JITStack) {
  delete unwrap(JITStack);
}
